@@ -24,31 +24,51 @@
 # FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 # DEALINGS IN THE SOFTWARE.
 
-import ast
-import code
 import optparse
-import os
-import readline
+import code
+import ast
 import sys
 
 import hy
 
+from hy.importer import ast_compile, import_buffer_to_module
 from hy.lex.states import Idle, LexException
 from hy.lex.machine import Machine
 from hy.compiler import hy_compile
-from hy.core import process
-from hy.importer import ast_compile, import_buffer_to_module
+from hy.completer import completion
 
-import hy.completer
-
-from hy.macros import macro
+from hy.macros import macro, require, process
 from hy.models.expression import HyExpression
 from hy.models.string import HyString
 from hy.models.symbol import HySymbol
 
-
 _machine = Machine(Idle, 1, 0)
 
+try:
+    import __builtin__ as builtins
+except ImportError:
+    import builtins
+
+
+class HyQuitter(object):
+    def __init__(self, name):
+        self.name = name
+
+    def __repr__(self):
+        return "Use (%s) or Ctrl-D (i.e. EOF) to exit" % (self.name)
+
+    __str__ = __repr__
+
+    def __call__(self, code=None):
+        try:
+            sys.stdin.close()
+        except:
+            pass
+        raise SystemExit(code)
+
+builtins.quit = HyQuitter('quit')
+builtins.exit = HyQuitter('exit')
+
 
 class HyREPL(code.InteractiveConsole):
     def runsource(self, source, filename='<input>', symbol='single'):
@@ -66,7 +86,7 @@ class HyREPL(code.InteractiveConsole):
             return True
 
         try:
-            tokens = process(_machine.nodes)
+            tokens = process(_machine.nodes, "__console__")
         except Exception:
             _machine = Machine(Idle, 1, 0)
             self.showtraceback()
@@ -74,7 +94,7 @@ class HyREPL(code.InteractiveConsole):
 
         _machine = Machine(Idle, 1, 0)
         try:
-            _ast = hy_compile(tokens, root=ast.Interactive)
+            _ast = hy_compile(tokens, "__console__", root=ast.Interactive)
             code = ast_compile(_ast, filename, symbol)
         except Exception:
             self.showtraceback()
@@ -135,6 +155,9 @@ def ideas_macro():
 
 """)])
 
+require("hy.cmdline", "__console__")
+require("hy.cmdline", "__main__")
+
 
 def run_command(source):
     try:
@@ -157,23 +180,15 @@ def run_repl(hr=None):
     sys.ps1 = "=> "
     sys.ps2 = "... "
 
-    history = os.path.expanduser("~/.hy-history")
-    readline.parse_and_bind("set blink-matching-paren on")
+    with completion():
+        if not hr:
+            hr = HyREPL()
 
-    try:
-        readline.read_history_file(history)
-    except IOError:
-        open(history, 'a').close()
+        hr.interact("{appname} {version}".format(
+            appname=hy.__appname__,
+            version=hy.__version__
+        ))
 
-    readline.parse_and_bind("tab: complete")
-
-    if not hr:
-        hr = HyREPL()
-    hr.interact("{appname} {version}".format(
-        appname=hy.__appname__,
-        version=hy.__version__
-    ))
-    readline.write_history_file(history)
     return 0
 
 
