@@ -1,4 +1,5 @@
 # Copyright (c) 2013 Paul Tagliamonte <paultag@debian.org>
+# Copyright (c) 2013 Julien Danjou <julien@danjou.info>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -25,12 +26,3 @@ if sys.version_info[0] >= 3:
     str_type = str
 else:
     str_type = unicode
-
-
-def flatten_literal_list(entry):
-    for e in entry:
-        if type(e) == list:
-            for x in flatten_literal_list(e):
-                yield x  # needs more yield-from
-        else:
-            yield e
